@@ -8,22 +8,28 @@ from abc import ABCMeta, abstractmethod
 from argparse import ArgumentTypeError
 from ast import literal_eval
 from collections import OrderedDict
+from textwrap import dedent
 
 from six import add_metaclass
 
 from virtualenv.discovery.cached_py_info import LogCmd
 from virtualenv.info import WIN_CPYTHON_2
-from virtualenv.pyenv_cfg import PyEnvCfg
 from virtualenv.util.path import Path, safe_delete
 from virtualenv.util.six import ensure_str, ensure_text
 from virtualenv.util.subprocess import run_cmd
-from virtualenv.util.zipapp import ensure_file_on_disk
 from virtualenv.version import __version__
 
+from .pyenv_cfg import PyEnvCfg
+
 HERE = Path(os.path.abspath(__file__)).parent
 DEBUG_SCRIPT = HERE / "debug.py"
 
 
+class CreatorMeta(object):
+    def __init__(self):
+        self.error = None
+
+
 @add_metaclass(ABCMeta)
 class Creator(object):
     """A class that given a python Interpreter creates a virtual environment"""
@@ -38,8 +44,10 @@ class Creator(object):
         self._debug = None
         self.dest = Path(options.dest)
         self.clear = options.clear
+        self.no_vcs_ignore = options.no_vcs_ignore
         self.pyenv_cfg = PyEnvCfg.from_folder(self.dest)
-        self.app_data = options.app_data.folder
+        self.app_data = options.app_data
+        self.env = options.env
 
     def __repr__(self):
         return ensure_str(self.__unicode__())
@@ -51,6 +59,7 @@ class Creator(object):
         return [
             ("dest", ensure_text(str(self.dest))),
             ("clear", self.clear),
+            ("no_vcs_ignore", self.no_vcs_ignore),
         ]
 
     @classmethod
@@ -68,11 +77,14 @@ class Creator(object):
         """Add CLI arguments for the creator.
 
         :param parser: the CLI parser
+        :param app_data: the application data folder
         :param interpreter: the interpreter we're asked to create virtual environment for
         :param meta: value as returned by :meth:`can_create`
         """
         parser.add_argument(
-            "dest", help="directory to create virtualenv at", type=cls.validate_dest,
+            "dest",
+            help="directory to create virtualenv at",
+            type=cls.validate_dest,
         )
         parser.add_argument(
             "--clear",
@@ -81,6 +93,13 @@ class Creator(object):
             help="remove the destination directory if exist before starting (will overwrite files otherwise)",
             default=False,
         )
+        parser.add_argument(
+            "--no-vcs-ignore",
+            dest="no_vcs_ignore",
+            action="store_true",
+            help="don't create VCS ignore directive in the destination directory",
+            default=False,
+        )
 
     @abstractmethod
     def create(self):
@@ -94,7 +113,7 @@ class Creator(object):
         def non_write_able(dest, value):
             common = Path(*os.path.commonprefix([value.parts, dest.parts]))
             raise ArgumentTypeError(
-                "the destination {} is not write-able at {}".format(dest.relative_to(common), common)
+                "the destination {} is not write-able at {}".format(dest.relative_to(common), common),
             )
 
         # the file system must be able to encode
@@ -113,13 +132,15 @@ class Creator(object):
         if refused:
             raise ArgumentTypeError(
                 "the file system codec ({}) cannot handle characters {!r} within {!r}".format(
-                    encoding, "".join(refused.keys()), raw_value
-                )
+                    encoding,
+                    "".join(refused.keys()),
+                    raw_value,
+                ),
             )
         if os.pathsep in raw_value:
             raise ArgumentTypeError(
                 "destination {!r} must not contain the path separator ({}) as this would break "
-                "the activation scripts".format(raw_value, os.pathsep)
+                "the activation scripts".format(raw_value, os.pathsep),
             )
 
         value = Path(raw_value)
@@ -149,6 +170,8 @@ class Creator(object):
             safe_delete(self.dest)
         self.create()
         self.set_pyenv_cfg()
+        if not self.no_vcs_ignore:
+            self.setup_ignore_vcs()
 
     def set_pyenv_cfg(self):
         self.pyenv_cfg.content = OrderedDict()
@@ -157,13 +180,32 @@ class Creator(object):
         self.pyenv_cfg["version_info"] = ".".join(str(i) for i in self.interpreter.version_info)
         self.pyenv_cfg["virtualenv"] = __version__
 
+    def setup_ignore_vcs(self):
+        """Generate ignore instructions for version control systems."""
+        # mark this folder to be ignored by VCS, handle https://www.python.org/dev/peps/pep-0610/#registered-vcs
+        git_ignore = self.dest / ".gitignore"
+        if not git_ignore.exists():
+            git_ignore.write_text(
+                dedent(
+                    """
+                    # created by virtualenv automatically
+                    *
+                    """,
+                ).lstrip(),
+            )
+        # Mercurial - does not support the .hgignore file inside a subdirectory directly, but only if included via the
+        # subinclude directive from root, at which point on might as well ignore the directory itself, see
+        # https://www.selenic.com/mercurial/hgignore.5.html for more details
+        # Bazaar - does not support ignore files in sub-directories, only at root level via .bzrignore
+        # Subversion - does not support ignore files, requires direct manipulation with the svn tool
+
     @property
     def debug(self):
         """
         :return: debug information about the virtual environment (only valid after :meth:`create` has run)
         """
         if self._debug is None and self.exe is not None:
-            self._debug = get_env_debug_info(self.exe, self.debug_script(), self.app_data)
+            self._debug = get_env_debug_info(self.exe, self.debug_script(), self.app_data, self.env)
         return self._debug
 
     # noinspection PyMethodMayBeStatic
@@ -171,11 +213,11 @@ class Creator(object):
         return DEBUG_SCRIPT
 
 
-def get_env_debug_info(env_exe, debug_script, app_data):
-    env = os.environ.copy()
+def get_env_debug_info(env_exe, debug_script, app_data, env):
+    env = env.copy()
     env.pop(str("PYTHONPATH"), None)
 
-    with ensure_file_on_disk(debug_script, app_data) as debug_script:
+    with app_data.ensure_extracted(debug_script) as debug_script:
         cmd = [str(env_exe), str(debug_script)]
         if WIN_CPYTHON_2:
             cmd = [ensure_text(i) for i in cmd]
@@ -185,7 +227,13 @@ def get_env_debug_info(env_exe, debug_script, app_data):
     # noinspection PyBroadException
     try:
         if code != 0:
-            result = literal_eval(out)
+            if out:
+                result = literal_eval(out)
+            else:
+                if code == 2 and "file" in err:
+                    # Re-raise FileNotFoundError from `run_cmd()`
+                    raise OSError(err)
+                raise Exception(err)
         else:
             result = json.loads(out)
         if err:
