@@ -1,7 +1,6 @@
 from __future__ import absolute_import, unicode_literals
 
 import os
-import pipes
 import re
 import shutil
 import subprocess
@@ -11,12 +10,17 @@ from os.path import dirname, normcase
 import pytest
 import six
 
-from virtualenv.info import IS_PYPY, WIN_CPYTHON_2
+from virtualenv.info import IS_PYPY, PY2, WIN_CPYTHON_2
 from virtualenv.run import cli_run
 from virtualenv.util.path import Path
 from virtualenv.util.six import ensure_str, ensure_text
 from virtualenv.util.subprocess import Popen
 
+if PY2:
+    from pipes import quote
+else:
+    from shlex import quote
+
 
 class ActivationTester(object):
     def __init__(self, of_class, session, cmd, activate_script, extension):
@@ -31,13 +35,17 @@ class ActivationTester(object):
         self.pydoc_call = "pydoc -w pydoc_test"
         self.script_encoding = "utf-8"
         self._version = None
+        self.unix_line_ending = True
 
     def get_version(self, raise_on_fail):
         if self._version is None:
             # locally we disable, so that contributors don't need to have everything setup
             try:
                 process = Popen(
-                    self._version_cmd, universal_newlines=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE,
+                    self._version_cmd,
+                    universal_newlines=True,
+                    stdout=subprocess.PIPE,
+                    stderr=subprocess.PIPE,
                 )
                 out, err = process.communicate()
                 result = out if out else err
@@ -63,6 +71,16 @@ class ActivationTester(object):
 
     def __call__(self, monkeypatch, tmp_path):
         activate_script = self._creator.bin_dir / self.activate_script
+
+        # check line endings are correct type
+        script_content = activate_script.read_bytes()
+        for line in script_content.split(b"\n")[:-1]:
+            cr = b"\r" if sys.version_info.major == 2 else 13
+            if self.unix_line_ending:
+                assert line == b"" or line[-1] != cr, script_content.decode("utf-8")
+            else:
+                assert line[-1] == cr, script_content.decode("utf-8")
+
         test_script = self._generate_test_script(activate_script, tmp_path)
         monkeypatch.chdir(ensure_text(str(tmp_path)))
 
@@ -112,6 +130,7 @@ class ActivationTester(object):
             self.activate_call(activate_script),
             self.print_python_exe(),
             self.print_os_env_var("VIRTUAL_ENV"),
+            self.print_prompt(),
             # \\ loads documentation from the virtualenv site packages
             self.pydoc_call,
             self.deactivate,
@@ -129,7 +148,12 @@ class ActivationTester(object):
         expected = self._creator.exe.parent / os.path.basename(sys.executable)
         assert self.norm_path(out[2]) == self.norm_path(expected), raw
         assert self.norm_path(out[3]) == self.norm_path(self._creator.dest).replace("\\\\", "\\"), raw
-        assert out[4] == "wrote pydoc_test.html", raw
+        # Some attempts to test the prompt output print more than 1 line.
+        # So we need to check if the prompt exists on any of them.
+        prompt_text = "({}) ".format(self._creator.env_name)
+        assert any(prompt_text in line for line in out[4:-3]), raw
+
+        assert out[-3] == "wrote pydoc_test.html", raw
         content = tmp_path / "pydoc_test.html"
         assert content.exists(), raw
         # post deactivation, same as before
@@ -137,7 +161,7 @@ class ActivationTester(object):
         assert out[-1] == "None", raw
 
     def quote(self, s):
-        return pipes.quote(s)
+        return quote(s)
 
     def python_cmd(self, cmd):
         return "{} -c {}".format(os.path.basename(sys.executable), self.quote(cmd))
@@ -153,10 +177,14 @@ class ActivationTester(object):
         val = '"{}"'.format(var)
         return self.python_cmd(
             "import os; import sys; v = os.environ.get({}); print({})".format(
-                val, "v" if six.PY3 or IS_PYPY else "None if v is None else v.decode(sys.getfilesystemencoding())",
+                val,
+                "v" if six.PY3 or IS_PYPY else "None if v is None else v.decode(sys.getfilesystemencoding())",
             ),
         )
 
+    def print_prompt(self):
+        return NotImplemented
+
     def activate_call(self, script):
         cmd = self.quote(ensure_text(str(self.activate_cmd)))
         scr = self.quote(ensure_text(str(script)))
@@ -188,7 +216,10 @@ class RaiseOnNonSourceCall(ActivationTester):
     def __call__(self, monkeypatch, tmp_path):
         env, activate_script = super(RaiseOnNonSourceCall, self).__call__(monkeypatch, tmp_path)
         process = Popen(
-            self.non_source_activate(activate_script), stdout=subprocess.PIPE, stderr=subprocess.PIPE, env=env,
+            self.non_source_activate(activate_script),
+            stdout=subprocess.PIPE,
+            stderr=subprocess.PIPE,
+            env=env,
         )
         out, _err = process.communicate()
         err = _err.decode("utf-8")
